#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import QUB
from qubtree import Node
import CurveFitting

# Curve-fitting without nonlinear regression.

class LinRegFitter:
    def __init__(self):
        self.paramsNode = Node('Params')
        param = self.paramsNode['Param']
        param['a'].data = 1.0
        param['b'].data = 1.0
        self.__a = param['a'].data
        self.__b = param['b'].data
        self.data = Node('FitData')
        self.data['X'].data = 1.0
        self.data['Y'].data = 1.0
        self.__n = 1

    def setNumComponents(self, n):
        pass # what, you want to fit it to _two_ lines?

    def getNumComponents(self):
        return 1

    def useData(self, dataset):
        self.data = dataset
        self.__n = min(dataset['X'].data.count, dataset['Y'].data.count)
        # here we should set some reasonable starting vals, but for linreg just fitting is fine
        self.fit(1, 0)
        
    def calcY(self, comp_ix, x, nextX=0.0):
        return self.__a[0] + x * self.__b[0]

    def getEqn(self):
        return 'b * x + a'
    
    def promptOptions(self):
        pass

    def getCurves(self, curves):
        return CurveFitting.RebuildCurves(self, curves)

    def fit(self, maxiter, precision=0.01): # , iterCallback=lambda x:x, doPrintStats=1):
        X = self.data['X'].data
        Y = self.data['Y'].data
        n = self.__n
        sumX = sumXX = sumY = sumYY = sumXY = 0
        for x in X:
            sumX = sumX + x
            sumXX = sumXX + x * x
        for y in Y:
            sumY = sumY + y
            sumYY = sumYY + y * y
        for i in range(len(X)):
            sumXY = sumXY + X[i] * Y[i]
            
        Sxx = sumXX - sumX * sumX / n
        Syy = sumYY - sumY * sumY / n
        Sxy = sumXY - sumX * sumY / n

        b = Sxy / Sxx
        a = (sumY - b * sumX) / n

        self.__a[0] = a
        self.__b[0] = b

CurveFitting.fitters['Linear Regression'] = LinRegFitter
